%%% @author Tony Rogvall <tony@rogvall.se>
%%% @copyright (C) 2012, Tony Rogvall
%%% @doc
%%%    GSM 03.38 codec
%%% @end
%%% Created : 16 Oct 2012 by Tony Rogvall <tony@rogvall.se>

-module(gsms_0338).

-export([decode/1, decode_char/1]).
-export([encode/1, encode/2, encode_char/1]).

decode(Bin) when is_binary(Bin) ->
    decode_list(binary_to_list(Bin));
decode(Cs) when is_list(Cs) ->
    decode_list(Cs).

%% 
decode_list([16#1B,C|Cs]) ->
    case C of
	16#0A -> [16#000C|decode_list(Cs)];  %%  FORM FEED
	16#14 -> [16#005E|decode_list(Cs)];  %%  CIRCUMFLEX ACCENT
	16#28 -> [16#007B|decode_list(Cs)];  %%  LEFT CURLY BRACKET
	16#29 -> [16#007D|decode_list(Cs)];  %%  RIGHT CURLY BRACKET
	16#2F -> [16#005C|decode_list(Cs)];  %%  REVERSE SOLIDUS
	16#3C -> [16#005B|decode_list(Cs)];  %%  LEFT SQUARE BRACKET
	16#3D -> [16#007E|decode_list(Cs)];  %%  TILDE
	16#3E -> [16#005D|decode_list(Cs)];  %%  RIGHT SQUARE BRACKET
	16#40 -> [16#007C|decode_list(Cs)];  %%  VERTICAL LINE
	16#65 -> [16#20AC|decode_list(Cs)];  %%  EURO SIGN    
	_ -> [16#A0|decode_list(Cs)]
    end;
decode_list([C|Cs]) ->
    [decode_char(C)|decode_list(Cs)];
decode_list([]) ->
    [].

decode_char(C) ->
    case C band 16#7f of
	16#00 -> 16#0040;	%%	COMMERCIAL AT
	%%16#00  16#0000  %%  NULL
	16#01 -> 16#00A3; %%  POUND SIGN
	16#02 -> 16#0024; %%  DOLLAR SIGN
	16#03 -> 16#00A5; %%  YEN SIGN
	16#04 -> 16#00E8; %%  LATIN SMALL LETTER E WITH GRAVE
	16#05 -> 16#00E9; %%  LATIN SMALL LETTER E WITH ACUTE
	16#06 -> 16#00F9; %%  LATIN SMALL LETTER U WITH GRAVE
	16#07 -> 16#00EC; %%  LATIN SMALL LETTER I WITH GRAVE
	16#08 -> 16#00F2; %%  LATIN SMALL LETTER O WITH GRAVE
	16#09 -> 16#00E7; %%  LATIN SMALL LETTER C WITH CEDILLA
	%%16#09  16#00C7  %%  LATIN CAPITAL LETTER C WITH CEDILLA
	16#0A -> 16#000A; %%  LINE FEED
	16#0B -> 16#00D8; %%  LATIN CAPITAL LETTER O WITH STROKE
	16#0C -> 16#00F8; %%  LATIN SMALL LETTER O WITH STROKE
	16#0D -> 16#000D; %%  CARRIAGE RETURN
	16#0E -> 16#00C5; %%  LATIN CAPITAL LETTER A WITH RING ABOVE
	16#0F -> 16#00E5; %%  LATIN SMALL LETTER A WITH RING ABOVE
	16#10 -> 16#0394; %%  GREEK CAPITAL LETTER DELTA
	16#11 -> 16#005F; %%  LOW LINE
	16#12 -> 16#03A6; %%  GREEK CAPITAL LETTER PHI
	16#13 -> 16#0393; %%  GREEK CAPITAL LETTER GAMMA
	16#14 -> 16#039B; %%  GREEK CAPITAL LETTER LAMDA
	16#15 -> 16#03A9; %%  GREEK CAPITAL LETTER OMEGA
	16#16 -> 16#03A0; %%  GREEK CAPITAL LETTER PI
	16#17 -> 16#03A8; %%  GREEK CAPITAL LETTER PSI
	16#18 -> 16#03A3; %%  GREEK CAPITAL LETTER SIGMA
	16#19 -> 16#0398; %%  GREEK CAPITAL LETTER THETA
	16#1A -> 16#039E; %%  GREEK CAPITAL LETTER XI
	16#1C -> 16#00C6; %%  LATIN CAPITAL LETTER AE
	16#1D -> 16#00E6; %%  LATIN SMALL LETTER AE
	16#1E -> 16#00DF; %%  LATIN SMALL LETTER SHARP S (German)
	16#1F -> 16#00C9; %%  LATIN CAPITAL LETTER E WITH ACUTE
	16#20 -> 16#0020; %%  SPACE
	16#21 -> 16#0021; %%  EXCLAMATION MARK
	16#22 -> 16#0022; %%  QUOTATION MARK
	16#23 -> 16#0023; %%  NUMBER SIGN
	16#24 -> 16#00A4; %%  CURRENCY SIGN
	16#25 -> 16#0025; %%  PERCENT SIGN
	16#26 -> 16#0026; %%  AMPERSAND
	16#27 -> 16#0027; %%  APOSTROPHE
	16#28 -> 16#0028; %%  LEFT PARENTHESIS
	16#29 -> 16#0029; %%  RIGHT PARENTHESIS
	16#2A -> 16#002A; %%  ASTERISK
	16#2B -> 16#002B; %%  PLUS SIGN
	16#2C -> 16#002C; %%  COMMA
	16#2D -> 16#002D; %%  HYPHEN-MINUS
	16#2E -> 16#002E; %%  FULL STOP
	16#2F -> 16#002F; %%  SOLIDUS
	16#30 -> 16#0030; %%  DIGIT ZERO
	16#31 -> 16#0031; %%  DIGIT ONE
	16#32 -> 16#0032; %%  DIGIT TWO
	16#33 -> 16#0033; %%  DIGIT THREE
	16#34 -> 16#0034; %%  DIGIT FOUR
	16#35 -> 16#0035; %%  DIGIT FIVE
	16#36 -> 16#0036; %%  DIGIT SIX
	16#37 -> 16#0037; %%  DIGIT SEVEN
	16#38 -> 16#0038; %%  DIGIT EIGHT
	16#39 -> 16#0039; %%  DIGIT NINE
	16#3A -> 16#003A; %%  COLON
	16#3B -> 16#003B; %%  SEMICOLON
	16#3C -> 16#003C; %%  LESS-THAN SIGN
	16#3D -> 16#003D; %%  EQUALS SIGN
	16#3E -> 16#003E; %%  GREATER-THAN SIGN
	16#3F -> 16#003F; %%  QUESTION MARK
	16#40 -> 16#00A1; %%  INVERTED EXCLAMATION MARK
	16#41 -> 16#0041; %%  LATIN CAPITAL LETTER A
	%%16#41  16#0391  %%  GREEK CAPITAL LETTER ALPHA
	16#42 -> 16#0042; %%  LATIN CAPITAL LETTER B
	%%16#42  16#0392  %%  GREEK CAPITAL LETTER BETA
	16#43 -> 16#0043; %%  LATIN CAPITAL LETTER C
	16#44 -> 16#0044; %%  LATIN CAPITAL LETTER D
	16#45 -> 16#0045; %%  LATIN CAPITAL LETTER E
	%%16#45  16#0395  %%  GREEK CAPITAL LETTER EPSILON
	16#46 -> 16#0046; %%  LATIN CAPITAL LETTER F
	16#47 -> 16#0047; %%  LATIN CAPITAL LETTER G
	16#48 -> 16#0048; %%  LATIN CAPITAL LETTER H
	%%16#48  16#0397  %%  GREEK CAPITAL LETTER ETA
	16#49 -> 16#0049; %%  LATIN CAPITAL LETTER I
	%%16#49  16#0399  %%  GREEK CAPITAL LETTER IOTA
	16#4A -> 16#004A; %%  LATIN CAPITAL LETTER J
	16#4B -> 16#004B; %%  LATIN CAPITAL LETTER K
	%%16#4B  16#039A  %%  GREEK CAPITAL LETTER KAPPA
	16#4C -> 16#004C; %%  LATIN CAPITAL LETTER L
	16#4D -> 16#004D; %%  LATIN CAPITAL LETTER M
	%%16#4D  16#039C  %%  GREEK CAPITAL LETTER MU
	16#4E -> 16#004E; %%  LATIN CAPITAL LETTER N
	%%16#4E  16#039D  %%  GREEK CAPITAL LETTER NU
	16#4F -> 16#004F; %%  LATIN CAPITAL LETTER O
	%%16#4F  16#039F  %%  GREEK CAPITAL LETTER OMICRON
	16#50 -> 16#0050; %%  LATIN CAPITAL LETTER P
	%%16#50  16#03A1  %%  GREEK CAPITAL LETTER RHO
	16#51 -> 16#0051; %%  LATIN CAPITAL LETTER Q
	16#52 -> 16#0052; %%  LATIN CAPITAL LETTER R
	16#53 -> 16#0053; %%  LATIN CAPITAL LETTER S
	16#54 -> 16#0054; %%  LATIN CAPITAL LETTER T
	%%16#54  16#03A4  %%  GREEK CAPITAL LETTER TAU
	16#55 -> 16#0055; %%  LATIN CAPITAL LETTER U
	16#56 -> 16#0056; %%  LATIN CAPITAL LETTER V
	16#57 -> 16#0057; %%  LATIN CAPITAL LETTER W
	16#58 -> 16#0058; %%  LATIN CAPITAL LETTER X
	%%16#58  16#03A7  %%  GREEK CAPITAL LETTER CHI
	16#59 -> 16#0059; %%  LATIN CAPITAL LETTER Y
	%%16#59  16#03A5  %%  GREEK CAPITAL LETTER UPSILON
	16#5A -> 16#005A; %%  LATIN CAPITAL LETTER Z
	%%16#5A  16#0396  %%  GREEK CAPITAL LETTER ZETA
	16#5B -> 16#00C4; %%  LATIN CAPITAL LETTER A WITH DIAERESIS
	16#5C -> 16#00D6; %%  LATIN CAPITAL LETTER O WITH DIAERESIS
	16#5D -> 16#00D1; %%  LATIN CAPITAL LETTER N WITH TILDE
	16#5E -> 16#00DC; %%  LATIN CAPITAL LETTER U WITH DIAERESIS
	16#5F -> 16#00A7; %%  SECTION SIGN
	16#60 -> 16#00BF; %%  INVERTED QUESTION MARK
	16#61 -> 16#0061; %%  LATIN SMALL LETTER A
	16#62 -> 16#0062; %%  LATIN SMALL LETTER B
	16#63 -> 16#0063; %%  LATIN SMALL LETTER C
	16#64 -> 16#0064; %%  LATIN SMALL LETTER D
	16#65 -> 16#0065; %%  LATIN SMALL LETTER E
	16#66 -> 16#0066; %%  LATIN SMALL LETTER F
	16#67 -> 16#0067; %%  LATIN SMALL LETTER G
	16#68 -> 16#0068; %%  LATIN SMALL LETTER H
	16#69 -> 16#0069; %%  LATIN SMALL LETTER I
	16#6A -> 16#006A; %%  LATIN SMALL LETTER J
	16#6B -> 16#006B; %%  LATIN SMALL LETTER K
	16#6C -> 16#006C; %%  LATIN SMALL LETTER L
	16#6D -> 16#006D; %%  LATIN SMALL LETTER M
	16#6E -> 16#006E; %%  LATIN SMALL LETTER N
	16#6F -> 16#006F; %%  LATIN SMALL LETTER O
	16#70 -> 16#0070; %%  LATIN SMALL LETTER P
	16#71 -> 16#0071; %%  LATIN SMALL LETTER Q
	16#72 -> 16#0072; %%  LATIN SMALL LETTER R
	16#73 -> 16#0073; %%  LATIN SMALL LETTER S
	16#74 -> 16#0074; %%  LATIN SMALL LETTER T
	16#75 -> 16#0075; %%  LATIN SMALL LETTER U
	16#76 -> 16#0076; %%  LATIN SMALL LETTER V
	16#77 -> 16#0077; %%  LATIN SMALL LETTER W
	16#78 -> 16#0078; %%  LATIN SMALL LETTER X
	16#79 -> 16#0079; %%  LATIN SMALL LETTER Y
	16#7A -> 16#007A; %%  LATIN SMALL LETTER Z
	16#7B -> 16#00E4; %%  LATIN SMALL LETTER A WITH DIAERESIS
	16#7C -> 16#00F6; %%  LATIN SMALL LETTER O WITH DIAERESIS
	16#7D -> 16#00F1; %%  LATIN SMALL LETTER N WITH TILDE
	16#7E -> 16#00FC; %%  LATIN SMALL LETTER U WITH DIAERESIS
	16#7F -> 16#00E0  %%  LATIN SMALL LETTER A WITH GRAVE
    end.

encode(Data) ->
    encode(Data, -1).

encode(Bin,Max) when is_binary(Bin), is_integer(Max) ->
    encode_list(binary_to_list(Bin), Max);
encode(Cs,Max) when is_list(Cs), is_integer(Max) ->
    encode_list(Cs,Max).


encode_list(Cs,Max) ->
    encode_list_(Cs,Max,[]).

encode_list_(Cs,0,Acc) ->
    {iolist_to_binary(lists:reverse(Acc)),Cs};
encode_list_(Cs1=[C|Cs],I,Acc) ->
    try encode_char(C) of
	[Esc,Y] ->
	    if I >= 2 -> encode_list_(Cs,I-2,[Y,Esc|Acc]);
	       true -> {iolist_to_binary(lists:reverse(Acc)),Cs1}
	    end;
	Y -> encode_list_(Cs,I-1,[Y|Acc])
    catch
	error:_ ->
	    encode_list_(Cs,I-2,[16#A0,16#1B|Acc])
    end;
encode_list_([],_I,Acc) ->
    {iolist_to_binary(lists:reverse(Acc)),[]}.


encode_char(C) ->
    case C of
	16#0040 -> 16#00; %%  COMMERCIAL AT
	16#0000 -> 16#00; %%  NULL
	16#00A3 -> 16#01; %%  POUND SIGN
	16#0024 -> 16#02; %%  DOLLAR SIGN
	16#00A5 -> 16#03; %%  YEN SIGN
	16#00E8 -> 16#04; %%  LATIN SMALL LETTER E WITH GRAVE
	16#00E9 -> 16#05; %%  LATIN SMALL LETTER E WITH ACUTE
	16#00F9 -> 16#06; %%  LATIN SMALL LETTER U WITH GRAVE
	16#00EC -> 16#07; %%  LATIN SMALL LETTER I WITH GRAVE
	16#00F2 -> 16#08; %%  LATIN SMALL LETTER O WITH GRAVE
	16#00E7 -> 16#09; %%  LATIN SMALL LETTER C WITH CEDILLA
	16#00C7 -> 16#09; %%  LATIN CAPITAL LETTER C WITH CEDILLA
	16#000A -> 16#0A; %%  LINE FEED
	16#00D8 -> 16#0B; %%  LATIN CAPITAL LETTER O WITH STROKE
	16#00F8 -> 16#0C; %%  LATIN SMALL LETTER O WITH STROKE
	16#000D -> 16#0D; %%  CARRIAGE RETURN
	16#00C5 -> 16#0E; %%  LATIN CAPITAL LETTER A WITH RING ABOVE
	16#00E5 -> 16#0F; %%  LATIN SMALL LETTER A WITH RING ABOVE
	16#0394 -> 16#10; %%  GREEK CAPITAL LETTER DELTA
	16#005F -> 16#11; %%  LOW LINE
	16#03A6 -> 16#12; %%  GREEK CAPITAL LETTER PHI
	16#0393 -> 16#13; %%  GREEK CAPITAL LETTER GAMMA
	16#039B -> 16#14; %%  GREEK CAPITAL LETTER LAMDA
	16#03A9 -> 16#15; %%  GREEK CAPITAL LETTER OMEGA
	16#03A0 -> 16#16; %%  GREEK CAPITAL LETTER PI
	16#03A8 -> 16#17; %%  GREEK CAPITAL LETTER PSI
	16#03A3 -> 16#18; %%  GREEK CAPITAL LETTER SIGMA
	16#0398 -> 16#19; %%  GREEK CAPITAL LETTER THETA
	16#039E -> 16#1A; %%  GREEK CAPITAL LETTER XI
	16#00C6 -> 16#1C; %%  LATIN CAPITAL LETTER AE
	16#00E6 -> 16#1D; %%  LATIN SMALL LETTER AE
	16#00DF -> 16#1E; %%  LATIN SMALL LETTER SHARP S (German)
	16#00C9 -> 16#1F; %%  LATIN CAPITAL LETTER E WITH ACUTE
	16#0020 -> 16#20; %%  SPACE
	16#0021 -> 16#21; %%  EXCLAMATION MARK
	16#0022 -> 16#22; %%  QUOTATION MARK
	16#0023 -> 16#23; %%  NUMBER SIGN
	16#00A4 -> 16#24; %%  CURRENCY SIGN
	16#0025 -> 16#25; %%  PERCENT SIGN
	16#0026 -> 16#26; %%  AMPERSAND
	16#0027 -> 16#27; %%  APOSTROPHE
	16#0028 -> 16#28; %%  LEFT PARENTHESIS
	16#0029 -> 16#29; %%  RIGHT PARENTHESIS
	16#002A -> 16#2A; %%  ASTERISK
	16#002B -> 16#2B; %%  PLUS SIGN
	16#002C -> 16#2C; %%  COMMA
	16#002D -> 16#2D; %%  HYPHEN-MINUS
	16#002E -> 16#2E; %%  FULL STOP
	16#002F -> 16#2F; %%  SOLIDUS
	16#0030 -> 16#30; %%  DIGIT ZERO
	16#0031 -> 16#31; %%  DIGIT ONE
	16#0032 -> 16#32; %%  DIGIT TWO
	16#0033 -> 16#33; %%  DIGIT THREE
	16#0034 -> 16#34; %%  DIGIT FOUR
	16#0035 -> 16#35; %%  DIGIT FIVE
	16#0036 -> 16#36; %%  DIGIT SIX
	16#0037 -> 16#37; %%  DIGIT SEVEN
	16#0038 -> 16#38; %%  DIGIT EIGHT
	16#0039 -> 16#39; %%  DIGIT NINE
	16#003A -> 16#3A; %%  COLON
	16#003B -> 16#3B; %%  SEMICOLON
	16#003C -> 16#3C; %%  LESS-THAN SIGN
	16#003D -> 16#3D; %%  EQUALS SIGN
	16#003E -> 16#3E; %%  GREATER-THAN SIGN
	16#003F -> 16#3F; %%  QUESTION MARK
	16#00A1 -> 16#40; %%  INVERTED EXCLAMATION MARK
	16#0041 -> 16#41; %%  LATIN CAPITAL LETTER A
	16#0391 -> 16#41; %%  GREEK CAPITAL LETTER ALPHA
	16#0042 -> 16#42; %%  LATIN CAPITAL LETTER B
	16#0392 -> 16#42; %%  GREEK CAPITAL LETTER BETA
	16#0043 -> 16#43; %%  LATIN CAPITAL LETTER C
	16#0044 -> 16#44; %%  LATIN CAPITAL LETTER D
	16#0045 -> 16#45; %%  LATIN CAPITAL LETTER E
	16#0395 -> 16#45; %%  GREEK CAPITAL LETTER EPSILON
	16#0046 -> 16#46; %%  LATIN CAPITAL LETTER F
	16#0047 -> 16#47; %%  LATIN CAPITAL LETTER G
	16#0048 -> 16#48; %%  LATIN CAPITAL LETTER H
	16#0397 -> 16#48; %%  GREEK CAPITAL LETTER ETA
	16#0049 -> 16#49; %%  LATIN CAPITAL LETTER I
	16#0399 -> 16#49; %%  GREEK CAPITAL LETTER IOTA
	16#004A -> 16#4A; %%  LATIN CAPITAL LETTER J
	16#004B -> 16#4B; %%  LATIN CAPITAL LETTER K
	16#039A -> 16#4B; %%  GREEK CAPITAL LETTER KAPPA
	16#004C -> 16#4C; %%  LATIN CAPITAL LETTER L
	16#004D -> 16#4D; %%  LATIN CAPITAL LETTER M
	16#039C -> 16#4D; %%  GREEK CAPITAL LETTER MU
	16#004E -> 16#4E; %%  LATIN CAPITAL LETTER N
	16#039D -> 16#4E; %%  GREEK CAPITAL LETTER NU
	16#004F -> 16#4F; %%  LATIN CAPITAL LETTER O
	16#039F -> 16#4F; %%  GREEK CAPITAL LETTER OMICRON
	16#0050 -> 16#50; %%  LATIN CAPITAL LETTER P
	16#03A1 -> 16#50; %%  GREEK CAPITAL LETTER RHO
	16#0051 -> 16#51; %%  LATIN CAPITAL LETTER Q
	16#0052 -> 16#52; %%  LATIN CAPITAL LETTER R
	16#0053 -> 16#53; %%  LATIN CAPITAL LETTER S
	16#0054 -> 16#54; %%  LATIN CAPITAL LETTER T
	16#03A4 -> 16#54; %%  GREEK CAPITAL LETTER TAU
	16#0055 -> 16#55; %%  LATIN CAPITAL LETTER U
	16#0056 -> 16#56; %%  LATIN CAPITAL LETTER V
	16#0057 -> 16#57; %%  LATIN CAPITAL LETTER W
	16#0058 -> 16#58; %%  LATIN CAPITAL LETTER X
	16#03A7 -> 16#58; %%  GREEK CAPITAL LETTER CHI
	16#0059 -> 16#59; %%  LATIN CAPITAL LETTER Y
	16#03A5 -> 16#59; %%  GREEK CAPITAL LETTER UPSILON
	16#005A -> 16#5A; %%  LATIN CAPITAL LETTER Z
	16#0396 -> 16#5A; %%  GREEK CAPITAL LETTER ZETA
	16#00C4 -> 16#5B; %%  LATIN CAPITAL LETTER A WITH DIAERESIS
	16#00D6 -> 16#5C; %%  LATIN CAPITAL LETTER O WITH DIAERESIS
	16#00D1 -> 16#5D; %%  LATIN CAPITAL LETTER N WITH TILDE
	16#00DC -> 16#5E; %%  LATIN CAPITAL LETTER U WITH DIAERESIS
	16#00A7 -> 16#5F; %%  SECTION SIGN
	16#00BF -> 16#60; %%  INVERTED QUESTION MARK
	16#0061 -> 16#61; %%  LATIN SMALL LETTER A
	16#0062 -> 16#62; %%  LATIN SMALL LETTER B
	16#0063 -> 16#63; %%  LATIN SMALL LETTER C
	16#0064 -> 16#64; %%  LATIN SMALL LETTER D
	16#0065 -> 16#65; %%  LATIN SMALL LETTER E
	16#0066 -> 16#66; %%  LATIN SMALL LETTER F
	16#0067 -> 16#67; %%  LATIN SMALL LETTER G
	16#0068 -> 16#68; %%  LATIN SMALL LETTER H
	16#0069 -> 16#69; %%  LATIN SMALL LETTER I
	16#006A -> 16#6A; %%  LATIN SMALL LETTER J
	16#006B -> 16#6B; %%  LATIN SMALL LETTER K
	16#006C -> 16#6C; %%  LATIN SMALL LETTER L
	16#006D -> 16#6D; %%  LATIN SMALL LETTER M
	16#006E -> 16#6E; %%  LATIN SMALL LETTER N
	16#006F -> 16#6F; %%  LATIN SMALL LETTER O
	16#0070 -> 16#70; %%  LATIN SMALL LETTER P
	16#0071 -> 16#71; %%  LATIN SMALL LETTER Q
	16#0072 -> 16#72; %%  LATIN SMALL LETTER R
	16#0073 -> 16#73; %%  LATIN SMALL LETTER S
	16#0074 -> 16#74; %%  LATIN SMALL LETTER T
	16#0075 -> 16#75; %%  LATIN SMALL LETTER U
	16#0076 -> 16#76; %%  LATIN SMALL LETTER V
	16#0077 -> 16#77; %%  LATIN SMALL LETTER W
	16#0078 -> 16#78; %%  LATIN SMALL LETTER X
	16#0079 -> 16#79; %%  LATIN SMALL LETTER Y
	16#007A -> 16#7A; %%  LATIN SMALL LETTER Z
	16#00E4 -> 16#7B; %%  LATIN SMALL LETTER A WITH DIAERESIS
	16#00F6 -> 16#7C; %%  LATIN SMALL LETTER O WITH DIAERESIS
	16#00F1 -> 16#7D; %%  LATIN SMALL LETTER N WITH TILDE
	16#00FC -> 16#7E; %%  LATIN SMALL LETTER U WITH DIAERESIS
	16#00E0 -> 16#7F; %%  LATIN SMALL LETTER A WITH GRAVE
	%% extensions
	16#000C -> [16#1B,16#0A];  %%  FORM FEED
	16#005E -> [16#1B,16#14];  %%  CIRCUMFLEX ACCENT
	16#007B -> [16#1B,16#28];  %%  LEFT CURLY BRACKET
	16#007D -> [16#1B,16#29];  %%  RIGHT CURLY BRACKET
	16#005C -> [16#1B,16#2F];  %%  REVERSE SOLIDUS
	16#005B -> [16#1B,16#3C];  %%  LEFT SQUARE BRACKET
	16#007E -> [16#1B,16#3D];  %%  TILDE
	16#005D -> [16#1B,16#3E];  %%  RIGHT SQUARE BRACKET
	16#007C -> [16#1B,16#40];  %%  VERTICAL LINE
	16#20AC -> [16#1B,16#65]   %%  EURO SIGN    
    end.
